/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/BasicImpl/MessageCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::MessageCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::MessageCallbackImpl class.
 */

#include "MessageCallbackImpl.h"

// For any localclass or message types, include the header and factory
// *implementation* needed to create and access the local class
#include <RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/Includes.h>
#include <TENA/UniqueID/Includes.h>
#include <TENA/Time/Includes.h>
#include <TENA/CoordinatedUniversalTime/Includes.h>
#include <TENA/Date/Includes.h>
#include <TENA/GpsTime/Includes.h>
#include <TENA/UnixTime/Includes.h>
#include <RTTC/EnvironmentalEffects/DisturbanceHistoryRecord/Includes.h>
#include <RTTC/EnvironmentalEffects/DisturbanceOffset/Includes.h>


#include <string>
#include <exception>
#include <iostream>
#include <iomanip>

///////////////////////////////////////////////////////////////////////////////
RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::
MessageCallbackImpl::
~MessageCallbackImpl()
{
  // Basic Implementation has no required behavior in state change callback
  // destructor.  Might need to free resources, close database connections,
  // etc.
}

///////////////////////////////////////////////////////////////////////////////
void
RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::
MessageCallbackImpl::
execute()
{
  /*!
    Messages are pushed onto the list in the basic implementation. This
    means that you have an ever-growing list.  The corresponding code in
    the main loop clears accumulated messages after "processing" them
    (in this example simply noting that a message was received).
   */
  _pCallbackInfo->getReceivedMessageList().push_back( _pMessage );

  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    // Use the ostream in the callback info to print
    std::ostream & os = _pCallbackInfo->getOutputStream();

    os << "A RTTC::EnvironmentalEffects::DisturbanceHistoryMessage was received" << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'r' << std::flush;
  }

  if ( 2 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << std::setw(0) << " " << "this.sequenceNumber ";
    os << _pMessage->get_sequenceNumber() << std::endl;
    os << std::setw(0) << " " << "this.platformID ";
    TENA::UniqueID::Pointer platformID_LocalClass_Ptr(
      _pMessage->get_platformID() );

    os << std::endl;
    os << std::setw(4) << " " << "platformID.siteID ";
    os << platformID_LocalClass_Ptr->get_siteID() << std::endl;
    os << std::setw(4) << " " << "platformID.applicationID ";
    os << platformID_LocalClass_Ptr->get_applicationID() << std::endl;
    os << std::setw(4) << " " << "platformID.objectID ";
    os << platformID_LocalClass_Ptr->get_objectID() << std::endl;
    os << std::setw(0) << " " << "this.timeStamp ";
    TENA::Time::Pointer timeStamp_LocalClass_Ptr(
      _pMessage->get_timeStamp() );

    os << std::endl;
    os << std::setw(0) << " " << "this.histories ";
    std::vector< RTTC::EnvironmentalEffects::DisturbanceHistoryRecord::Pointer > const & histories_Vector(
      _pMessage->get_histories() );
    os << std::endl;
    size_t histories_length = histories_Vector.size();
    for (size_t histories_index = 0;
         histories_index < histories_length;
         ++histories_index)
    {
      os << std::setw(4) << " " << "histories [ " << histories_index << " ]";
      os << std::endl;
      RTTC::EnvironmentalEffects::DisturbanceHistoryRecord::Pointer const & histories_element( histories_Vector[histories_index] );
      os << std::setw(8) << " " << ".xyzOffset ";
      RTTC::EnvironmentalEffects::DisturbanceOffset::Pointer xyzOffset_LocalClass_Ptr(
        histories_element->get_xyzOffset() );

      os << std::endl;
      os << std::setw(12) << " " << "xyzOffset.x ";
      os << xyzOffset_LocalClass_Ptr->get_x() << std::endl;
      os << std::setw(12) << " " << "xyzOffset.y ";
      os << xyzOffset_LocalClass_Ptr->get_y() << std::endl;
      os << std::setw(12) << " " << "xyzOffset.z ";
      os << xyzOffset_LocalClass_Ptr->get_z() << std::endl;
      os << std::setw(8) << " " << ".milliseconds ";
      os << histories_element->get_milliseconds() << std::endl;
    }
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception--i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}

///////////////////////////////////////////////////////////////////////////////
RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::
MessageCallbackImpl::
MessageCallbackImpl(
  RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::Pointer const & pMessage,
  CallbackInfoPtr const & pCallbackInfo )
  :
  _pCallbackInfo( pCallbackInfo ),
  _pMessage( pMessage )
{
  // Basic MessageCallback implementation requires no behavior at callback
  // creation.  This is fairly typical--most callback constructor will not have
  // behavior
}
