/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/BasicImpl/MessageCallbackImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::MessageCallbackImpl.
 *
 * This file contains the declaration of and documentation for the
 * RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::BasicImpl::MessageCallbackImpl class.
 */

#ifndef RTTC_EnvironmentalEffects_DisturbanceHistoryMessage_BasicImpl_MessageCallbackImpl_h
#define RTTC_EnvironmentalEffects_DisturbanceHistoryMessage_BasicImpl_MessageCallbackImpl_h

#include <DCT/System/config.h>
#include <basicImpl_RTTC_EnvironmentalEffects_v1_EXPORT.h>
#include <DCT/Utils/SmartPtr.h>
#include <TENA/Middleware/Callback/Callback.h>
#include <RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/Interface.h>
#include "CallbackInfo.h"

namespace RTTC
{
  namespace EnvironmentalEffects
  {
    namespace DisturbanceHistoryMessage
    {
      namespace BasicImpl
      {
        /*!\class MessageCallbackImpl MessageCallbackImpl.h \
           RTTC/EnvironmentalEffects/DisturbanceHistoryMessage/BasicImpl/MessageCallbackImpl.h
         *
         * \brief Callback created when a message is received.
         *
         * When a message is received, a MessageCallback is created.  This
         * basic implementation will push received messages onto a list,
         * which means you will have an ever-growing list.  A strategy to
         * remove this messages is required within the application.
         */
        class basicImpl_RTTC_EnvironmentalEffects_v1_EXPORT MessageCallbackImpl
          : public virtual TENA::Middleware::Callback::Callback
        {
        public:
          //! This is the only constructor.
          /*!
            \param[in] pMessage Pointer to the Message that was received.
            \param[in] pCallbackInfo The CallbackInfo object.

            Since this class is defined by the user, the constructor parameters
            can be whatever is needed by the application.  The message will
            almost always be one parameter.  Additional parameters are at the
            user's discretion.
          */
          MessageCallbackImpl(
            RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::Pointer const & pMessage,
            CallbackInfoPtr const & pCallbackInfo );

          //! Instances are destructed after invoking the execute() method.
          ~MessageCallbackImpl();

          //! Perform the behavior associated with this callback.
          /*!
            This method implements the user defined behavior that is necessary
            when receiving a message.  If the processing requires access to
            external variables, they should be provided to the constructor
            when this callback object is created.

            \note Since the user is free to invoke session::evokeCallback() in
            multiple threads concurrently, the execute method should be MT-safe.
          */
          void
          execute();

        private:
          //! Default copy construction is disallowed.
          MessageCallbackImpl( MessageCallbackImpl const & rhs );
          //! Default copy assignment is disallowed.
          MessageCallbackImpl & operator=( MessageCallbackImpl const & rhs );

          //! Hold a CallbackInfo
          CallbackInfoPtr _pCallbackInfo;

          //! Hold a reference to the received message
          RTTC::EnvironmentalEffects::DisturbanceHistoryMessage::Pointer _pMessage;
        };
      } // End of namespace BasicImpl
    } // End of namespace DisturbanceHistoryMessage
  } // End of namespace EnvironmentalEffects
} // End of namespace RTTC

#endif // ifndef RTTC_EnvironmentalEffects_DisturbanceHistoryMessage_BasicImpl_MessageCallbackImpl_h
