//////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/BasicImpl/DestructionCallbackFactoryImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * TENA::Platform::BasicImpl::DestructionCallbackFactoryImpl.
 *
 * This file contains the declaration of and documentation for the
 * TENA::Platform::BasicImpl::DestructionCallbackFactoryImpl class.
 */

#ifndef TENA_Platform_BasicImpl_DestructionCallbackFactoryImpl_h
#define TENA_Platform_BasicImpl_DestructionCallbackFactoryImpl_h

#include <DCT/System/config.h>
#include <basicImpl_TENA_Platform_v3_1_EXPORT.h>
#include <TENA/Platform/DestructionCallbackFactory.h>

#include "DestructionCallbackImpl.h"
#include "CallbackInfo.h"

namespace TENA
{
  namespace Platform
  {
    namespace BasicImpl
    {
      /*!\class DestuctionCallbackFactoryImpl DestructionCallbackFactoryImpl.h \
         TENA/Platform/BasicImpl/DestructionCallbackFactoryImpl.h
       *
       * \brief A Basic concrete implementation of an
       * TENA::Platform::DestructionCallbackFactory.
       *
       * This basic implementation uses the CallbackInfo to remove destroyed SDOs from
       * the list of discovered SDOs
       */
      class basicImpl_TENA_Platform_v3_1_EXPORT DestructionCallbackFactoryImpl
        : public virtual TENA::Platform::DestructionCallbackFactory
      {
      public:

        /*!
          The basic implementation constructor takes a single argument -- the CallbackInfo
          with a list of discovered TENA::Platform-s, (used by discovery and
          destruction callbacks), and a std::ostream used by the stateChange callback.
        */
        DestructionCallbackFactoryImpl( CallbackInfoPtr const & );

        /*!
          The basic implementation destructor has no behavior.  It is conceivable that
          some applications require behavior here, but most will probably not.
        */
        ~DestructionCallbackFactoryImpl();

        //! Create a Callback and return it to the TENA Middleware.
        /*!
          \param[in] pProxy A ProxyPtr for the new TENA::Platform::Servant that
          matches a current subscription has been detected by the TENA Middleware.

          \return An object that implements the TENA::Middleware::Callback::Callback
          interface.  Ownership of the memory is passed to the invoker.

          This method is invoked by the TENA Middleware when a new instance of an
          TENA::Platform::Servant that matches a subscription request is
          detected.  The returned Callback is placed on an internal queue, with
          callbacks removed from the queue and executed in response to user calls to
          session::evokeCallback()
        */
        std::auto_ptr< TENA::Middleware::Callback::Callback >
        create( TENA::Platform::ProxyPtr const & pProxy );

      private:
        //! Hold a CallbackInfo to communicate between main pgm and callbacks
        CallbackInfoPtr _pCallbackInfo;

        //! Default copy construction is disallowed.
        DestructionCallbackFactoryImpl(
          DestructionCallbackFactoryImpl const & rhs );
        //! Default copy assignment is disallowed.
        DestructionCallbackFactoryImpl & operator=(
          DestructionCallbackFactoryImpl const & rhs );
      };
    } // End of namespace BasicImpl
  } // End of namespace Platform
} // End of namespace TENA

#endif // ifndef TENA_Platform_BasicImpl_DestructionCallbackFactoryImpl_h
