/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/BasicImpl/DestructionCallbackImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * TENA::Platform::BasicImpl::DestructionCallbackImpl.
 *
 * This file contains the definition of the methods of the
 * TENA::Platform::BasicImpl::DestructionCallbackImpl class.
 */

#include "DestructionCallbackImpl.h"

// Proxy.h includes the PublicationState header to access SDO's state, if
// needed.
#include <TENA/Platform/Proxy.h>
#include <string>
#include <exception>
#include <iostream>

///////////////////////////////////////////////////////////////////////////////
TENA::Platform::BasicImpl::
DestructionCallbackImpl::
~DestructionCallbackImpl()
{
  // Basic Implementation has no required behavior in callback
  // destructor. Might need to free resources, close database connections, etc.
}

///////////////////////////////////////////////////////////////////////////////
TENA::Platform::BasicImpl::
DestructionCallbackImpl::
DestructionCallbackImpl(
  TENA::Platform::ProxyPtr const & pProxy,
  CallbackInfoPtr const & pCallbackInfo
)
  :
  _pCallbackInfo( pCallbackInfo ),
  _pProxy( pProxy )
{
  // Basic DestructionCallback implementation requires no behavior at callback
  // creation. This is fairly typical -- most callback constructor will not have
  // behavior
}


///////////////////////////////////////////////////////////////////////////////
void
TENA::Platform::BasicImpl::
DestructionCallbackImpl::
execute()
{
  _pCallbackInfo->getDiscoveredSDOlist().remove( _pProxy );

  if ( 1 < _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << "TENA::Platform::BasicImpl DestructionCallbackImpl execute()"
       << std::endl;
    os << "SDO ID " << _pProxy->getSDOid() << std::endl;
    os << "Version " << _pProxy->getPublicationState()->get_TENA_Platform_VersionCounter() << std::endl;
  }
  else if ( 1 == _pCallbackInfo->getVerbosity() )
  {
    std::ostream & os = _pCallbackInfo->getOutputStream();
    os << 'x' << std::flush;
  }

  // Callbacks should only catch exceptions if they can actually HANDLE the
  // exception -- i.e. correct the error condition.  Otherwise, the exception
  // will propagate up to the evokeCallbacks invocation where it may be handled
  // in an application specific manner.
}
