/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/DisturbanceServer/SubscriptionInfoImpl.cpp
 * \version 5.2.1
 *
 * \brief Contains the definition of methods of
 * TENA::Platform::DisturbanceServer::SubscriptionInfoImpl.
 *
 * This file contains the definition of the methods of the
 * TENA::Platform::DisturbanceServer::SubscriptionInfoImpl class.
 */

#include "SubscriptionInfoImpl.h"
#include "CallbackInfo.h" // To give smart pointer complete type for deletion

#include "DestructionCallbackFactoryImpl.h"
#include <TENA/Platform/NullDestructionCallbackFactory.h>
#include "DiscoveryCallbackFactoryImpl.h"
#include <TENA/Platform/NullDiscoveryCallbackFactory.h>
#include "StateChangeCallbackFactoryImpl.h"
#include <TENA/Platform/NullStateChangeCallbackFactory.h>


///////////////////////////////////////////////////////////////////////////////
TENA::Platform::DisturbanceServer::
SubscriptionInfoImpl::
SubscriptionInfoImpl()
:
  TENA::Platform::SubscriptionInfo(
    DiscoveryCallbackFactoryPtr(new NullDiscoveryCallbackFactory),
    StateChangeCallbackFactoryPtr(new NullStateChangeCallbackFactory),
    DestructionCallbackFactoryPtr(new NullDestructionCallbackFactory)    )
{
}

///////////////////////////////////////////////////////////////////////////////
TENA::Platform::DisturbanceServer::
SubscriptionInfoImpl::
SubscriptionInfoImpl( CallbackInfoPtr const & pCallbackInfo )
:
  TENA::Platform::SubscriptionInfo(
    DiscoveryCallbackFactoryPtr(
      new DiscoveryCallbackFactoryImpl( pCallbackInfo ) ),
    StateChangeCallbackFactoryPtr(
      new StateChangeCallbackFactoryImpl( pCallbackInfo ) ),
    DestructionCallbackFactoryPtr(
      new DestructionCallbackFactoryImpl( pCallbackInfo ) )   )
{
}

///////////////////////////////////////////////////////////////////////////////
TENA::Platform::DisturbanceServer::
SubscriptionInfoImpl::
~SubscriptionInfoImpl()
{
}
