/////////////////////////////////////////////////////////////////// -*- C++ -*-
/*! \file TENA/Platform/DisturbanceServer/SubscriptionInfoImpl.h
 * \version 5.2.1
 *
 * \brief Contains the declaration of and documentation for
 * TENA::Platform::DisturbanceServer::SubscriptionInfoImpl.
 *
 * This file contains the declaration of and documentation for the
 * TENA::Platform::DisturbanceServer::SubscriptionInfoImpl class.
 */

#ifndef TENA_Platform_DisturbanceServer_SubscriptionInfoImpl_h
#define TENA_Platform_DisturbanceServer_SubscriptionInfoImpl_h

#include <DCT/System/config.h>
#include <DisturbanceServer_v1_TENA_Platform_v3_1_EXPORT.h>
#include "CallbackInfo.h"
#include <TENA/Platform/SubscriptionInfo.h>
#include "DestructionCallbackFactoryImpl.h"
#include <TENA/Platform/NullDestructionCallbackFactory.h>
#include "DiscoveryCallbackFactoryImpl.h"
#include <TENA/Platform/NullDiscoveryCallbackFactory.h>
#include "StateChangeCallbackFactoryImpl.h"
#include <TENA/Platform/NullStateChangeCallbackFactory.h>


namespace TENA
{
  namespace Platform
  {
    namespace DisturbanceServer
    {
      /*!\class SubscriptionInfoImpl SubscriptionInfoImpl.h \
         TENA/Platform/DisturbanceServer/SubscriptionInfoImpl.h
       *
       * \brief Holder for all user-defined objects required to subscribe
       * to this SDO type.
       *
       * The basic implementation for this object should be sufficient
       * for most applications.
       *
       * Any custom variables needed by the callback factories can be
       * passed in through the CallbackInfo object.
       */
      class DisturbanceServer_v1_TENA_Platform_v3_1_EXPORT SubscriptionInfoImpl
        : public TENA::Platform::SubscriptionInfo
      {
      public:
        //! Use this default ctor if no callbacks are desired.
        /*!
         * This constructor can be used when creating SubscriptionInfo
         * objects that will be used to get a Proxy from an SDO pointer
         * that will only be used to invoke methods, and access to the
         * PublicationState is not desired.  In this case there is no
         * need to hook up the callback machinery.
         *
         */
        SubscriptionInfoImpl();

        /*!
         * This ctor creates callback factories using the provided
         * CallbackInfoPtr .
         *
         * Use this ctor when subscribing to this SDO type.
         */
        explicit SubscriptionInfoImpl( CallbackInfoPtr const & );

        virtual
        ~SubscriptionInfoImpl();

      private:
        //! Default copy construction is disallowed.
        SubscriptionInfoImpl(
          SubscriptionInfoImpl const & rhs );
        //! Default copy assignment is disallowed.
        SubscriptionInfoImpl & operator=(
          SubscriptionInfoImpl const & rhs );

      };
    } // End of namespace DisturbanceServer
  } // End of namespace Platform
} // End of namespace TENA

#endif // ifndef TENA_Platform_DisturbanceServer_SubscriptionInfoImpl_h
