#include "orientation.h"
#include <math.h>

void Orientation::Quaternian_to_Euler(double w,
                         double x,
                         double y,
                         double z,
                         double& yaw,
                         double& pitch,
                         double& roll)
{
  double w2 = w*w;
  double x2 = x*x;
  double y2 = y*y;
  double z2 = z*z;

  yaw = atan2((2.0 * (x*y + z*w)) , w2 + x2 - y2 - z2);
  pitch = asin(-2.0 * (x*z - y*w));
  roll = atan2((2.0 * (y*z + x*w)) , w2 - x2 - y2 + z2);  
}


void Orientation::Euler_to_Quaternian(double yaw,
                         double pitch,
                         double roll,
                         double& w,
                         double& x,
                         double& y,
                         double& z)
{
  double c1 = cos(yaw/2);
  double s1 = sin(yaw/2);
  double c2 = cos(pitch/2);
  double s2 = sin(pitch/2);
  double c3 = cos(roll/2);
  double s3 = sin(roll/2);

  w = c1*c2*c3 + s1*s2*s3;
  x = c1*c2*s3 - s1*s2*c3;
  y = c1*s2*c3 + s1*c2*s3;
  z = s1*c2*c3 - c1*s2*s3;
}


